const {$failOnCallbackArg, $ignoreResult} = require('../lib')
const {expect} = require('chai')
const sinon = require('sinon')

describe('lib', () => {
    describe('$failOnCallbackArg', () => {
        it('rejects if and only if the callbacks arg is truthy', () => {
            const asyncNoArgs = (arg1, arg2, callback) => {
                setTimeout(callback, 100)
            }
            const callWithArg = callback => callback('abc')
            const asyncOneArg = (arg1, arg2, callback) => {
                setTimeout(callWithArg(callback), 100)
            }
            const object = {asyncNoArgs, asyncOneArg}
            sinon.spy(object, 'asyncNoArgs')
            sinon.spy(object, 'asyncOneArg')
            const methodArgs = [1, 2]
            const $runTest = $failOnCallbackArg(object, 'asyncNoArgs', ...methodArgs)
                .then(() => {
                    expect(object.asyncNoArgs.calledWith(...methodArgs)).to.be.true
                    return $failOnCallbackArg(object, 'asyncOneArg', ...methodArgs)
                })
                .then(() => {
                    throw new Error('test should\'ve failed here')
                })
                .catch(error => {
                    expect(object.asyncOneArg.calledWith(...methodArgs)).to.be.true
                    expect(error).to.equal('abc')
                })

            return $runTest
        })
    })


    describe('ignoreResult', () => {
        it('executes a promise-getter and then returns the previous result', () => {
            const $firstPromise = Promise.resolve('cat')
            const promiseGetterSpy = sinon.spy()
            const $promiseGetter = () => {
                promiseGetterSpy()
                return Promise.resolve('dog')
            }

            const $runTest = $firstPromise
                .then($ignoreResult($promiseGetter))
                .then(result => {
                    expect(promiseGetterSpy.calledOnce).to.be.true
                    expect(result).to.equal('cat')
                })

            return $runTest
        })
    })
})